#Author: David Szybala (www.davidszybala.com)
#Title: outlinerOrganizer.py
#Date: 11/06/14
#Description: This script will organize the outliner by node type.  All nodes of the same type will be sorted alphabetically.
#Installation: Place this file in your maya scripts directory (e.g. ..\My Documents\maya\scripts) then run the following python commands:
#   import outlinerOrganizer
#   outlinerOrganizer.main()
import maya.cmds as cmds


class outlinerOrganizer():
    def __init__(self):
        self.uiOoNodeTypeList = None
        self.uiOoStatusTxt = None
        self.GUI()


    #************************************************UTILITY FUNCTIONS************************************************
    def getCurrentNodeType(self, currentNode):
        shapeChildren = []
        currentNodeType = None
        
        shapeChildren = cmds.listRelatives(currentNode, shapes=True, fullPath=True, noIntermediate=True)
        if shapeChildren != None:
            currentNodeType = cmds.nodeType(shapeChildren[0])
        else:
            currentNodeType = cmds.nodeType(currentNode)
        return currentNodeType


    def getAllNodeTypes(self):
        allDagObjects = cmds.ls(dag=True, long=True)
        allNodeTypes = []
        shapeChildren = []
        currentNodeType = None
        
        for currentObj in allDagObjects:
            shapeChildren = cmds.listRelatives(currentObj, shapes=True, fullPath=True, noIntermediate=True)
            if shapeChildren == None:
                currentNodeType = cmds.nodeType(currentObj)
                if not(currentNodeType in allNodeTypes):
                    allNodeTypes.append(currentNodeType)
        return allNodeTypes


    def getOrderByType(self, typeOrder, nodesToSort):
        nodesByType = []
        nodesSorted = []
        currentNodeType = None
        index = 0
        
        for i in range(0, len(typeOrder), 1):
            nodesByType.append([])
        for currentNode in nodesToSort:
            currentNodeType = self.getCurrentNodeType(currentNode)
            if currentNodeType in typeOrder:
                index = typeOrder.index(currentNodeType)
                nodesByType[index].append(currentNode)
            else:
                print("ERROR: Object " + currentNode + " has a type not in sort list: " + str(currentNodeType) + ".")
        for nodeList in nodesByType:
            if len(nodeList) > 0:
                nodeList.sort()
        for i in range(0, len(nodesByType), 1):
            for x in range(0, len(nodesByType[i]), 1):
                nodesSorted.append(nodesByType[i][x])
        return nodesSorted


    def setSortChildren(self, typeOrder, currentParent):
        allChildren = cmds.listRelatives(currentParent, children=True, fullPath=True, noIntermediate=True)
        
        if allChildren != None:
            if len(allChildren) > 1:
                allChildren = self.getOrderByType(typeOrder, allChildren)
                for currentChild in allChildren:
                    cmds.reorder(currentChild, back=True)
                    self.setSortChildren(typeOrder, currentChild)
            else:
                self.setSortChildren(typeOrder, allChildren[0])


    def setMoveListItem(self, moveDirection):
        currentItem = cmds.textScrollList(self.uiOoNodeTypeList, query=True, selectItem=True)
        currentItemIndex = cmds.textScrollList(self.uiOoNodeTypeList, query=True, selectIndexedItem=True)
        totalItems = cmds.textScrollList(self.uiOoNodeTypeList, query=True, numberOfItems=True)
        
        if currentItemIndex != None:
            currentItem = currentItem[0]
            currentItemIndex = currentItemIndex[0]
            if moveDirection == 1:
                cmds.textScrollList(self.uiOoNodeTypeList, edit=True, removeIndexedItem=(currentItemIndex))
                cmds.textScrollList(self.uiOoNodeTypeList, edit=True, appendPosition=[1, currentItem])
                cmds.textScrollList(self.uiOoNodeTypeList, edit=True, selectIndexedItem=1)
            elif moveDirection == 2 and currentItemIndex != 1:
                cmds.textScrollList(self.uiOoNodeTypeList, edit=True, removeIndexedItem=(currentItemIndex))
                cmds.textScrollList(self.uiOoNodeTypeList, edit=True, appendPosition=[(currentItemIndex - 1), currentItem])
                cmds.textScrollList(self.uiOoNodeTypeList, edit=True, selectIndexedItem=(currentItemIndex - 1))
            elif moveDirection == 3 and currentItemIndex != totalItems:
                cmds.textScrollList(self.uiOoNodeTypeList, edit=True, removeIndexedItem=(currentItemIndex))
                cmds.textScrollList(self.uiOoNodeTypeList, edit=True, appendPosition=[(currentItemIndex + 1), currentItem])
                cmds.textScrollList(self.uiOoNodeTypeList, edit=True, selectIndexedItem=(currentItemIndex + 1))
            else:
                cmds.textScrollList(self.uiOoNodeTypeList, edit=True, removeIndexedItem=(currentItemIndex))
                cmds.textScrollList(self.uiOoNodeTypeList, edit=True, append=(currentItem))
                cmds.textScrollList(self.uiOoNodeTypeList, edit=True, selectIndexedItem=(totalItems))


    #************************************************UI COMMANDS************************************************
    def setNodeTypeListUpdate(self, *args):
        allNodeTypes = self.getAllNodeTypes()
        
        cmds.textScrollList(self.uiOoNodeTypeList, edit=True, removeAll=True)
        for currentType in allNodeTypes:
            cmds.textScrollList(self.uiOoNodeTypeList, edit=True, append=(currentType))
        cmds.textScrollList(self.uiOoNodeTypeList, edit=True, selectIndexedItem=1)
        cmds.text(self.uiOoStatusTxt, edit=True, label="List updated. Standing by...")


    def setMoveItemTop(self, *args):
        self.setMoveListItem(1)


    def setMoveItemUp(self, *args):
        self.setMoveListItem(2)


    def setMoveItemDown(self, *args):
        self.setMoveListItem(3)


    def setMoveItemBottom(self, *args):
        self.setMoveListItem(4)


    def setOrganizeOutliner(self, *args):
        topAssemblies = cmds.ls(assemblies=True, long=True)
        typeOrder = cmds.textScrollList(self.uiOoNodeTypeList, query=True, allItems=True)
        
        cmds.text(self.uiOoStatusTxt, edit=True, label="Organizing outliner. Please wait...")
        cmds.refresh(currentView=True)
        topAssemblies = self.getOrderByType(typeOrder, topAssemblies)
        for currentObj in topAssemblies:
            cmds.reorder(currentObj, back=True)
            self.setSortChildren(typeOrder, currentObj)
        cmds.text(self.uiOoStatusTxt, edit=True, label="Outliner has been organized. Standing by...")


    #************************************************UI CREATION************************************************
    def GUI(self):
        if (cmds.window("uiOoWindow", exists=True)):
            cmds.deleteUI("uiOoWindow")
        ooWindow = cmds.window("uiOoWindow", title="Outliner Organizer", minimizeButton=True, maximizeButton=False, sizeable=False, resizeToFitChildren=True)
        ooFormMain = cmds.formLayout(numberOfDivisions=10)
        ooListColumn = cmds.columnLayout()
        self.uiOoNodeTypeList = cmds.textScrollList(width=150, numberOfRows=15, allowMultiSelection=False)
        cmds.setParent("..") #end columnLayout
        ooButtonColumn = cmds.columnLayout(rowSpacing=5)
        cmds.button(width=125, label="Update List", command=self.setNodeTypeListUpdate)
        cmds.text(height=22, label="")
        cmds.button(width=125, label="Move To Top", command=self.setMoveItemTop)
        cmds.button(width=125, label="Move Up", command=self.setMoveItemUp)
        cmds.button(width=125, label="Move Down", command=self.setMoveItemDown)
        cmds.button(width=125, label="Move To Bottom", command=self.setMoveItemBottom)
        cmds.text(height=23, label="")
        cmds.button(width=125, label="Organize Outliner Now", command=self.setOrganizeOutliner)
        cmds.setParent("..") #end columnLayout
        ooStatusColumn = cmds.columnLayout()
        self.uiOoStatusTxt = cmds.text(width=275, height=15, align="left", label="Standing by...")
        cmds.setParent("..") #end columnLayout
        cmds.formLayout(ooFormMain, edit=True, attachForm=[(ooListColumn, "top", 5), (ooListColumn, "left", 5), (ooListColumn, "bottom", 5), (ooButtonColumn, "top", 5), (ooButtonColumn, "right", 5), (ooStatusColumn, "left", 5)], attachControl=[ooButtonColumn, "left", 5, ooListColumn], attachPosition=[ooStatusColumn, "bottom", 5, 10])
        cmds.setParent("..") #end formLayout
        self.setNodeTypeListUpdate()
        cmds.showWindow(ooWindow)


def main():
    outlinerOrganizer()


if __name__ == "__main__":
    main()